@@ -1,5 +1,5 @@
 __all__ = [
-    "ds",
+    "ds", "ds_pd",
     "atr", "atr_pd",
     "bb",
     "bbw",
@@ -24,5 +24,5 @@ __all__ = [
 ]
 from .alan_ta import (
     ds, atr, bb, bbw, change, cross, crossover, crossunder, cum, dd, dmi, donchian, ema, highest, hma, lowest, pivothigh, pivotlow, rma, sma, stdev, supertrend, tr, wma, wpr,  rsi, 
-    atr_pd
+    atr_pd, ds_pd
 )
\ No newline at end of file
